
#include "Wave_Writer.hpp"

/* Library Copyright (C) 2003-2004 Shay Green. This library is free software;
you can redistribute it and/or modify it under the terms of the GNU Lesser
General Public License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version. This
library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
details. You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

#include <assert.h>
#include <stdlib.h>

const int header_size = 0x2C;

static void exit_with_error( const char* str ) {
	fprintf( stderr, "Error: %s\n", str );
	exit( EXIT_FAILURE );
}

Wave_Writer::Wave_Writer( long sample_rate, const char* filename ) :
	buf( new unsigned char [buf_size] ),
	file( fopen( filename, "wb" ) ),
	sample_count( 0 ),
	rate( sample_rate ),
	buf_pos( header_size )
{
	stereo( 0 );
	
	if ( !buf )
		exit_with_error( "Out of memory" );
	
	if ( !file )
		exit_with_error( "Couldn't open WAVE file for writing" );
}

void Wave_Writer::flush() {
	if ( buf_pos && !fwrite( buf, buf_pos, 1, file ) )
		exit_with_error( "Couldn't write WAVE data" );
	buf_pos = 0;
}

void Wave_Writer::write( const sample_t* in, long remain )
{
	sample_count += remain;
	while ( remain )
	{
		if ( buf_pos >= buf_size )
			flush();
		
		long n = (unsigned long) (buf_size - buf_pos) / sizeof (sample_t);
		if ( n > remain )
			n = remain;
		remain -= n;
		
		// convert to lsb first format
		unsigned char* p = &buf [buf_pos];
		while ( n-- ) {
			int s = *in++;
			*p++ = (unsigned char) s;
			*p++ = (unsigned char) (s >> 8);
		}
		
		buf_pos = p - buf;
		assert( buf_pos <= buf_size );
	}
}

Wave_Writer::~Wave_Writer()
{
	flush();
	
	// generate header
	long ds = sample_count * sizeof (sample_t);
	long rs = header_size - 8 + ds;
	int frame_size = chan_count * sizeof (sample_t);
	long bps = rate * frame_size;
	unsigned char header [header_size] = {
		'R','I','F','F',
		rs,rs>>8,           // length of rest of file
		rs>>16,rs>>24,
		'W','A','V','E',
		'f','m','t',' ',
		0x10,0,0,0,         // size of fmt chunk
		1,0,                // uncompressed format
		chan_count,0,       // channel count
		rate,rate >> 8,     // sample rate
		rate>>16,rate>>24,
		bps,bps>>8,         // bytes per second
		bps>>16,bps>>24,
		frame_size,0,       // bytes per sample frame
		16,0,               // bits per sample
		'd','a','t','a',
		ds,ds>>8,ds>>16,ds>>24// size of sample data
		// ...              // sample data
	};
	
	// write header
	fseek( file, 0, SEEK_SET );
	fwrite( header, sizeof header, 1, file );
	
	fclose( file );
	delete [] buf;
}

